﻿-- Author      : Deldinor of Elune
-- Create Date : 2/1/2011 7:31:49 AM

addonName = "DailyGrind";
addonFullName = "Daily Grind";

DailyGrind = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceConsole-3.0", "AceEvent-3.0");
DailyGrind.version = GetAddOnMetadata(addonName, "Version");

BINDING_HEADER_DAILYGRIND = addonFullName;

local titleColor = "FF70B7FF";
addonTitle = "|c"..titleColor..addonFullName.."|r";
addonAbbr = "|c"..titleColor.."[DG]|r";
commandColor = "FFFFC654";
local disabledText = "|cFFFF0000DISABLED|r";
local enabledText = "|cFF00FF00ENABLED|r";
maxQuests = 25;

function DailyGrind:OnInitialize()
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(addonName, self.Options)
	local configDialog = LibStub("AceConfigDialog-3.0")
	self.optionsFrames = {}
	self.optionsFrames.DailyGrind = configDialog:AddToBlizOptions(addonName, addonName, nil, "General")
	
    self:RegisterChatCommand("dailygrind", "ChatCommand");
    self:RegisterChatCommand("dg", "ChatCommand");
    self:InitializeSettings();
end

function DailyGrind:InitializeSettings()
	if DailyGrindSettings == nil then
		DailyGrindSettings = self.defaultSettings;
	end
	if DailyGrindQuests == nil then
		DailyGrindQuests = {};
	end
    if DailyGrindSettings.Version == nil or DailyGrindSettings.Version ~= self.version then		
		self:UpdateOldSettings();
		DailyGrindSettings.Version = self.version;
	end
end

function DailyGrind:UpdateOldSettings()
	-- Fix possible missing 1.1 settings
	if DailyGrindSettings.Blacklist == nil then
		DailyGrindSettings.Blacklist = {};
	end
	if DailyGrindSettings.Rewards == nil then
		DailyGrindSettings.Rewards = {};
	end
	
	-- Fix 1.2 settings
	if DailyGrind:GetBlacklistCount() > 0 then
		DailyGrind.Blacklist = self:FixOldList(DailyGrindSettings.Blacklist);
	end
	if DailyGrind:GetRewardListCount() > 0 then
		DailyGrind.Rewards = self:FixOldList(DailyGrindSettings.Rewards);
	end
	
	-- Fix 1.3 issues (if any)
	--- Strip quest levels that may have been appended
	local tempTable = {};
	for index, value in pairs(DailyGrindQuests) do
		tempTable[index] = value;
	end
	DailyGrindQuests = {};
	for index, value in pairs(tempTable) do
		DailyGrindQuests[self:SanitizeQuestTitle(index)] = value;
	end
	--- Strip quest levels from blacklist
	local questList = { strsplit("\n", self:Blacklist_ToString()) };
	self:Blacklist_Populate(questList);
end

DailyGrind.defaultSettings = {
	Enabled = true,
	Version = GetAddOnMetadata(addonName, "Version"),
	Blacklist = {},
	Rewards = {},
}

function DailyGrind:OnEnable()
    self:RegisterEvent("GOSSIP_SHOW");
    self:RegisterEvent("QUEST_DETAIL");
    self:RegisterEvent("QUEST_PROGRESS");
    self:RegisterEvent("QUEST_COMPLETE");
end

function DailyGrind:ChatCommand(input)
	local args = { strsplit(" ", input:trim(), 3) };
	local command = args[1]:lower();
	 
	if command == "" then
		self:ShowOptions();
	
	elseif command == "import" then
		-- TODO: Implement confirmation.
		self:Import();
		
	elseif command == "reset" then
		-- TODO: Implement confirmation.
		self:Reset();
		
  	elseif command == "enable" or command == "on" then
		self:SetEnabled(true);
		DEFAULT_CHAT_FRAME:AddMessage(addonAbbr.." "..self:GetStatus());
		
	elseif command == "disable" or command == "off" then
		self:SetEnabled(false);
		DEFAULT_CHAT_FRAME:AddMessage(addonAbbr.." "..self:GetStatus());
		
	elseif command == "toggle" or command == "tog" then
		self:Toggle();
		DEFAULT_CHAT_FRAME:AddMessage(addonAbbr.." "..self:GetStatus());
		
	elseif command == "blacklist" or command == "bl" then
		self:HandleBlacklist(args);
	
	elseif command == "rewardlist" or command == "rl" then
		self:HandleRewardList(args);

	else
		self:ShowHelp();
		
    end
end

function DailyGrind:Import()
    for index, value in pairs(KnownDailies) do
		DailyGrindQuests[index] = value;
    end
	DEFAULT_CHAT_FRAME:AddMessage(addonAbbr.." All known quests imported.");
end

function DailyGrind:Reset()
	DailyGrindQuests = {};
	DEFAULT_CHAT_FRAME:AddMessage(addonAbbr.." Quest history reset.");
end

function DailyGrind:Toggle()
	self:SetEnabled(not self:GetEnabled());
end

function DailyGrind:SetEnabled(state)
	DailyGrindSettings.Enabled = state;
end

function DailyGrind:GetEnabled()
	return DailyGrindSettings.Enabled;
end

function DailyGrind:GetStatus()
	if self:GetEnabled() then
		return addonTitle.." - "..enabledText;
	else
		return addonTitle.." - "..disabledText;
	end
end

function DailyGrind:GetNumPlayerActiveQuests()
	local numEntries, numQuests = GetNumQuestLogEntries();
	return numQuests;
end

function DailyGrind:GOSSIP_SHOW()
	if not DailyGrindSettings.Enabled then
		return;
	end

	-- Turn in known completed quests.
	local numActiveQuests = GetNumGossipActiveQuests();
	if numActiveQuests > 0 then
		local activeQuests = {GetGossipActiveQuests()};
		for i = 1, numActiveQuests, 1 do
			local titleIdx = (4 * (i - 1)) + 1;
			local isDailyIdx = titleIdx + 3;
			local questTitle = self:SanitizeQuestTitle(activeQuests[titleIdx]);
			if activeQuests[isDailyIdx] and DailyGrindQuests[questTitle] then
				if self:IsBlacklisted(questTitle) then
					DEFAULT_CHAT_FRAME:AddMessage(addonAbbr.." "..blacklistText..": Skipped "..questTitle);
				else
					SelectGossipActiveQuest(i);
				end
			end
		end
	end
	
	-- Accept known available quests.
	local numAvailableQuests = GetNumGossipAvailableQuests();
	if numAvailableQuests > 0 then
		local availableQuests = {GetGossipAvailableQuests()};
		for i = 1, numAvailableQuests, 1 do
			local titleIdx = (5 * (i - 1)) + 1;
			local questTitle = self:SanitizeQuestTitle(availableQuests[titleIdx]);
			if DailyGrindQuests[questTitle] then
				if self:IsBlacklisted(questTitle) then
					DEFAULT_CHAT_FRAME:AddMessage(addonAbbr.." "..blacklistText..": Skipped "..questTitle);
				else
					if self:GetNumPlayerActiveQuests() < maxQuests then
						SelectGossipAvailableQuest(i);
					else
						DEFAULT_CHAT_FRAME:AddMessage(addonAbbr.." Cannot auto-accept \""..questTitle.."\": Quest log is full.");
					end
				end
			end
		end
	end
	
end

function DailyGrind:QUEST_DETAIL()
	if not DailyGrindSettings.Enabled then
		return;
	end
	local questTitle = GetTitleText();
	if DailyGrindQuests[questTitle] and not self:IsBlacklisted(questTitle) then
		if self:GetNumPlayerActiveQuests() < maxQuests then
			AcceptQuest();
		else
			DEFAULT_CHAT_FRAME:AddMessage(addonAbbr.." Cannot auto-accept \""..questTitle.."\": Quest log is full.");
		end
	end
end

function DailyGrind:QUEST_PROGRESS()
	if not DailyGrindSettings.Enabled then
		return;
	end
	local questTitle = GetTitleText();
	if IsQuestCompletable() and DailyGrindQuests[questTitle] and not self:IsBlacklisted(questTitle) then
		CompleteQuest();
	end
end

function DailyGrind:QUEST_COMPLETE()
	local questTitle = GetTitleText();
	if DailyGrindSettings.Enabled and DailyGrindQuests[questTitle] and not self:IsBlacklisted(questTitle) then
		if GetDailyQuestsCompleted() < GetMaxDailyQuests() then
			if GetNumQuestChoices() == 0 then
				GetQuestReward(0);
			else
				self:SelectReward(questTitle);
			end
		else
			DEFAULT_CHAT_FRAME:AddMessage(addonAbbr.." Cannot auto-complete \""..questTitle.."\": Daily quest limit ("..GetMaxDailyQuests()..") reached. Go outside or something.");
		end
	elseif (QuestIsDaily() or QuestIsWeekly()) then
		DailyGrindQuests[questTitle] = 1;	-- always record, even when disabled
	end
end

function DailyGrind:SelectReward(questTitle)
	local myChoices = self:GetMatchingQuestRewards();
	local numMyChoices = self:TableCount(myChoices);
	if numMyChoices == 1 then	-- Exactly one matching item. Happy path.
		local choiceIndex, itemName;
		for index, value in pairs(myChoices) do
			choiceIndex = index;
			itemName = value;
		end
		DEFAULT_CHAT_FRAME:AddMessage(addonAbbr.." "..rewardListText..": \""..questTitle.."\" => \""..itemName.."\" selected.");
		GetQuestReward(choiceIndex);
	elseif numMyChoices > 1 then	-- 2+ matching items. Not so happy.
		DEFAULT_CHAT_FRAME:AddMessage(addonAbbr.." "..rewardListText..": \""..questTitle.."\" => Multiple matches found:\n");
		for index, value in pairs(myChoices) do	-- Build a string that displays all the conflicting rewards.
			DEFAULT_CHAT_FRAME:AddMessage(" - "..value);
		end
		DEFAULT_CHAT_FRAME:AddMessage("  Please choose your reward manually and then review your "..rewardListText..".\n");
	else	-- No matching items. Sad.
		DEFAULT_CHAT_FRAME:AddMessage(addonAbbr.." "..rewardListText..": \""..questTitle.."\" => No matching rewards found; please choose manually.");
	end
end
function DailyGrind:ShowHelp()
	DEFAULT_CHAT_FRAME:AddMessage("==]  "..self:GetStatus().."  [==\n");
	DEFAULT_CHAT_FRAME:AddMessage("Dailies that you complete are added to a database. Whenever you see that daily again, you will auto-accept/complete it.");
	DEFAULT_CHAT_FRAME:AddMessage("A list of currently-known dailies is included; you may import this list into your database using |c"..commandColor.."/dg import|r.");
	DEFAULT_CHAT_FRAME:AddMessage("Usage:  |c"..commandColor.."/dg <enable|on> <disable|off> <tog[gle]> <import> <reset>|r\n");
	DEFAULT_CHAT_FRAME:AddMessage(rewardListText..": Set automatic reward preferences (|c"..commandColor.."/dg rl help|r for more info)");
	DEFAULT_CHAT_FRAME:AddMessage(blacklistText..": Ignore specific quests (|c"..commandColor.."/dg bl help|r for more info)");
end
